package cc.fascinated.bat.features.scoresaber;

import cc.fascinated.bat.common.Colors;
import cc.fascinated.bat.common.DateUtils;
import cc.fascinated.bat.common.NumberUtils;
import cc.fascinated.bat.common.ScoreSaberUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.scoresaber.profiles.GuildNumberOneScoreFeedProfile;
import cc.fascinated.bat.features.scoresaber.profiles.GuildUserScoreFeedProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.beatsaber.scoresaber.ScoreSaberScoreToken;
import cc.fascinated.bat.service.DiscordService;
import cc.fascinated.bat.service.GuildService;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component @Log4j2
public class NumberOneScoreFeedListener implements EventListener {
    private final GuildService guildService;

    @Autowired
    public NumberOneScoreFeedListener(GuildService guildService) {
        this.guildService = guildService;
    }

    @Override
    public void onScoresaberScoreReceived(ScoreSaberPlayerScoreToken score, ScoreSaberLeaderboardToken leaderboard,
                                          ScoreSaberScoreToken.LeaderboardPlayerInfo player) {
        if (score.getScore().getRank() != 1) { // Only send if the score is a #1 score
            return;
        }
        if (!leaderboard.isRanked()) { // Only send if the leaderboard is ranked
            return;
        }

        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild == null) {
                continue;
            }
            GuildNumberOneScoreFeedProfile profile = batGuild.getProfile(GuildNumberOneScoreFeedProfile.class);
            if (profile == null || profile.getChannelId() == null) {
                continue;
            }

            TextChannel channel = profile.getAsTextChannel();
            if (channel == null) {
                log.error("Scoresaber user feed channel is null for guild {}, removing the stored channel.", guild.getId());
                profile.setChannelId(null);
                guildService.saveGuild(batGuild);
                continue;
            }
            channel.sendMessageEmbeds(this.buildScoreEmbed(score)).queue();
        }
    }

    /**
     * Builds an embed for a score.
     *
     * @param score The score.
     * @return The embed.
     */
    public MessageEmbed buildScoreEmbed(ScoreSaberPlayerScoreToken score) {
        ScoreSaberScoreToken scoreToken = score.getScore();
        ScoreSaberLeaderboardToken leaderboardToken = score.getLeaderboard();
        ScoreSaberScoreToken.LeaderboardPlayerInfo playerInfo = scoreToken.getLeaderboardPlayerInfo();
        return new EmbedBuilder()
                .setAuthor(playerInfo.getName() + " just set a new #1!", "https://scoresaber.com/u/%s".formatted(playerInfo.getId()),
                        "https://cdn.scoresaber.com/avatars/%s.jpg".formatted(playerInfo.getId()))
                .setDescription("**%s** (%s%s)\n[[Map Link]](%s) [[SS Profile]](%s)".formatted(
                        leaderboardToken.getSongName(),
                        ScoreSaberUtils.getFormattedDifficulty(leaderboardToken.getDifficulty().getDifficulty()),
                        leaderboardToken.isRanked() ? " " + leaderboardToken.getStars() + "⭐" : "",
                        "https://scoresaber.com/leaderboard/%s".formatted(leaderboardToken.getId()),
                        "https://scoresaber.com/u/%s".formatted(playerInfo.getId())
                ))
                .addField("Accuracy", "%s%%".formatted(
                        leaderboardToken.getMaxScore() == 0 ? "N/A" : NumberUtils.formatNumberCommas(((double) scoreToken.getBaseScore() / leaderboardToken.getMaxScore()) * 100)
                ), true)
                .addField("Raw PP", scoreToken.getPp() == 0 ? "Unranked" : NumberUtils.formatNumberCommas(scoreToken.getPp()), true)
                .addField("Rank", "#%s".formatted(NumberUtils.formatNumberCommas(scoreToken.getRank())), true)
                .addField("Misses", "%s".formatted(scoreToken.getMissedNotes()), true)
                .addField("Bad Cuts", "%s".formatted(scoreToken.getBadCuts()), true)
                .addField("Max Combo", "%s %s".formatted(
                        scoreToken.getMaxCombo(),
                        scoreToken.getMaxCombo() == leaderboardToken.getMaxScore() ? "(FC)" : ""
                ), true)
                .setColor(Colors.DEFAULT)
                .setTimestamp(DateUtils.getDateFromString(scoreToken.getTimeSet()).toInstant())
                .build();
    }
}
