package cc.fascinated.bat.features.scoresaber.command.numberone;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.TextChannelUtils;
import cc.fascinated.bat.features.scoresaber.profiles.GuildNumberOneScoreFeedProfile;
import cc.fascinated.bat.features.scoresaber.profiles.GuildUserScoreFeedProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.unions.GuildChannelUnion;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber-number-one-feed:channel.sub")
public class ChannelSubCommand extends BatSubCommand {
    private final GuildService guildService;

    @Autowired
    public ChannelSubCommand(GuildService guildService) {
        this.guildService = guildService;
    }

    @Override
    public void execute(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull TextChannel channel, @NonNull Member member, @NonNull SlashCommandInteraction interaction) {
        GuildNumberOneScoreFeedProfile profile = guild.getProfile(GuildNumberOneScoreFeedProfile.class);
        OptionMapping option = interaction.getOption("channel");
        if (option == null) {
            if (!TextChannelUtils.isValidChannel(profile.getChannelId())) {
                interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("Please provide a channel to set the ScoreSaber #1 feed channel to").build()).queue();
                return;
            }
            interaction.replyEmbeds(EmbedUtils.buildSuccessEmbed("The current ScoreSaber #1 feed channel is %s"
                    .formatted(TextChannelUtils.getChannelMention(profile.getChannelId()))).build()).queue();
            return;
        }

        GuildChannelUnion targetChannel = option.getAsChannel();
        if (targetChannel.getType() != ChannelType.TEXT) {
            interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("Invalid channel type, please provide a text channel").build()).queue();
            return;
        }

        profile.setChannelId(targetChannel.getId());
        guildService.saveGuild(guild);

        interaction.replyEmbeds(EmbedUtils.buildSuccessEmbed("Successfully set the ScoreSaber #1 feed channel to %s"
                .formatted(targetChannel.asTextChannel().getAsMention())).build()).queue();
    }
}
