package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.common.Colors;
import cc.fascinated.bat.common.DateUtils;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberUtils;
import cc.fascinated.bat.model.beatsaber.scoresaber.ScoreSaberAccountToken;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.features.scoresaber.profiles.UserScoreSaberProfile;
import cc.fascinated.bat.service.ScoreSaberService;
import lombok.NonNull;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import net.dv8tion.jda.internal.interactions.CommandDataImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class ScoreSaberCommand extends BatCommand {
    private final ScoreSaberService scoreSaberService;

    @Autowired
    public ScoreSaberCommand(@NonNull ScoreSaberService scoreSaberService, @NonNull ApplicationContext context) {
        super("scoresaber");
        super.setCategory(Category.BEAT_SABER);
        this.scoreSaberService = scoreSaberService;

        super.addSubCommand("link", context.getBean(LinkSubCommand.class));
        super.addSubCommand("user", context.getBean(UserSubCommand.class));
        super.addSubCommand("me", context.getBean(MeSubCommand.class));

        super.setDescription("General ScoreSaber commands");
        super.setCommandData(new CommandDataImpl(this.getName(), this.getDescription())
                .addSubcommands(new SubcommandData("link", "Link your ScoreSaber profile")
                        .addOptions(new OptionData(OptionType.STRING, "link", "Link your ScoreSaber profile", true))
                )
                .addSubcommands(new SubcommandData("user", "View a user's ScoreSaber profile")
                        .addOptions(new OptionData(OptionType.USER, "user", "The user to view the ScoreSaber profile of", true))
                )
                .addSubcommands(new SubcommandData("me", "View your ScoreSaber profile"))
        );
    }

    @Override
    public void execute(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull TextChannel channel, @NonNull Member member, @NonNull SlashCommandInteraction interaction) {
        sendProfileEmbed(true, user, scoreSaberService, interaction);
    }

    /**
     * Builds the profile embed for the ScoreSaber profile
     *
     * @param user The user to build the profile embed for
     * @param scoreSaberService The ScoreSaber service
     * @param interaction The interaction
     */
    public static void sendProfileEmbed(boolean isSelf, BatUser user, ScoreSaberService scoreSaberService, SlashCommandInteraction interaction) {
        UserScoreSaberProfile profile = user.getProfile(UserScoreSaberProfile.class);
        if (profile.getId() == null) {
            if (!isSelf) {
                interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("%s does not have a linked ScoreSaber account"
                        .formatted(user.getDiscordUser().getAsMention())).build()).queue();
            }
            interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("You do not have a linked ScoreSaber account").build()).queue();
            return;
        }

        // todo: handle rate limits
        ScoreSaberAccountToken account = scoreSaberService.getAccount(profile.getId());
        if (account == null) {
            if (!isSelf) {
                interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("%s has an invalid ScoreSaber account linked, please ask them to re-link their account"
                        .formatted(user.getDiscordUser().getAsMention())).build()).queue();
            }
            interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("You have an invalid ScoreSaber account linked, please re-link your account")
                    .build()).queue();
            return;
        }

        interaction.replyEmbeds(new EmbedBuilder()
                .setAuthor(account.getName() + "'s Profile", "https://scoresaber.com/u/%s".formatted(account.getId()),
                        "https://cdn.scoresaber.com/avatars/%s.jpg".formatted(account.getId()))
                .addField("Name", account.getName(), true)
                .addField("Country", account.getCountry(), true)
                .addField("Rank", "#" + NumberUtils.formatNumberCommas(account.getRank()), true)
                .addField("Country Rank", "#" + NumberUtils.formatNumberCommas(account.getCountryRank()), true)
                .addField("PP", NumberUtils.formatNumberCommas(account.getPp()), true)
                .addField("Joined", "<t:%s>".formatted(DateUtils.getDateFromString(account.getFirstSeen()).toInstant().toEpochMilli()/1000), true)
                .setTimestamp(LocalDateTime.now())
                .setColor(Colors.DEFAULT)
                .build()).queue();
    }
}
