package cc.fascinated.bat.features.scoresaber.command.userfeed;

import cc.fascinated.bat.command.BatCommand;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.interactions.commands.DefaultMemberPermissions;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import net.dv8tion.jda.internal.interactions.CommandDataImpl;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber-userfeed:feed.sub")
public class UserFeedCommand extends BatCommand {
    public UserFeedCommand(@NonNull ApplicationContext context) {
        super("scoresaber-user-feed");
        super.setCategory(Category.BEAT_SABER);

        super.addSubCommand("user", context.getBean(UserSubCommand.class));
        super.addSubCommand("channel", context.getBean(ChannelSubCommand.class));
        super.addSubCommand("clear-channel", context.getBean(ClearChannelSubCommand.class));
        super.addSubCommand("clear-users", context.getBean(ClearUsersSubCommand.class));

        super.setDescription("ScoreSaber user score feed commands");
        super.setCommandData(new CommandDataImpl(this.getName(), this.getDescription())
                .setGuildOnly(true)
                .addSubcommands(new SubcommandData("user", "Edit the users in the score feed")
                        .addOptions(new OptionData(OptionType.USER, "user", "Add or remove a user from the score feed", false))
                ).setDefaultPermissions(DefaultMemberPermissions.enabledFor(Permission.MANAGE_SERVER))

                .addSubcommands(new SubcommandData("clear-users", "Remove all users from the score feed"))
                .setDefaultPermissions(DefaultMemberPermissions.enabledFor(Permission.MANAGE_SERVER))

                .addSubcommands(new SubcommandData("channel", "Edit the channel the score feed is sent in")
                        .addOptions(new OptionData(OptionType.CHANNEL, "channel", "Set the channel to send the score feed in", false))
                ).setDefaultPermissions(DefaultMemberPermissions.enabledFor(Permission.MANAGE_SERVER))

                .addSubcommands(new SubcommandData("clear-channel", "Disable the score feed"))
                    .setDefaultPermissions(DefaultMemberPermissions.enabledFor(Permission.MANAGE_SERVER))
        );
    }
}
