package cc.fascinated.bat.features.scoresaber.command.userfeed;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.features.scoresaber.profiles.GuildUserScoreFeedProfile;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.features.scoresaber.profiles.UserScoreSaberProfile;
import cc.fascinated.bat.service.GuildService;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber-userfeed:user.sub")
public class UserSubCommand extends BatSubCommand {
    private final GuildService guildService;
    private final UserService userService;

    @Autowired
    public UserSubCommand(GuildService guildService, UserService userService) {
        this.guildService = guildService;
        this.userService = userService;
    }

    @Override
    public void execute(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull TextChannel channel, @NonNull Member member, @NonNull SlashCommandInteraction interaction) {
        GuildUserScoreFeedProfile profile = guild.getProfile(GuildUserScoreFeedProfile.class);
        OptionMapping option = interaction.getOption("user");
        if (option == null){
            if (profile.getTrackedUsers().isEmpty()) {
                interaction.replyEmbeds(EmbedUtils.buildGenericEmbed("There are no users being tracked in the ScoreSaber feed").build()).queue();
                return;
            }
            StringBuilder stringBuilder = new StringBuilder();
            for (String accountId : profile.getTrackedUsers()) {
                stringBuilder.append("[%s](%s)".formatted(
                        accountId,
                        "https://scoresaber.com/u/%s".formatted(accountId)
                ));
            }
            interaction.replyEmbeds(EmbedUtils.buildGenericEmbed("The current users being tracked in the ScoreSaber feed are:\n%s".formatted(stringBuilder.toString())).build()).queue();
            return;
        }

        User target = option.getAsUser();
        BatUser targetUser = userService.getUser(target.getId());
        UserScoreSaberProfile targetProfile = targetUser.getProfile(UserScoreSaberProfile.class);
        if (targetProfile.getId() == null) {
            interaction.replyEmbeds(EmbedUtils.buildErrorEmbed("The user you are trying to track does not have a linked ScoreSaber profile").build()).queue();
            return;
        }

        if (profile.getTrackedUsers().contains(targetProfile.getId())) {
            profile.getTrackedUsers().remove(targetProfile.getId());
            interaction.replyEmbeds(EmbedUtils.buildSuccessEmbed("Successfully removed %s from the ScoreSaber feed".formatted(target.getAsMention())).build()).queue();
        } else {
            profile.getTrackedUsers().add(targetProfile.getId());
            interaction.replyEmbeds(EmbedUtils.buildSuccessEmbed("Successfully added %s to the ScoreSaber feed".formatted(target.getAsMention())).build()).queue();
        }
        guildService.saveGuild(guild);
    }
}
