package cc.fascinated.bat.model;

import cc.fascinated.bat.common.Profile;
import cc.fascinated.bat.service.DiscordService;
import lombok.Getter;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import net.dv8tion.jda.api.entities.User;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@RequiredArgsConstructor
@Getter @Setter
@Document(collection = "users")
public class BatUser {

    /**
     * The ID of the user
     */
    @NonNull @Id private final String id;

    /**
     * The profiles for this user
     */
    private Map<String, Profile> profiles;

    /**
     * Gets the profile for the user
     *
     * @param clazz The class of the profile
     * @param <T> The type of the profile
     * @return The profile
     */
    public <T extends Profile> T getProfile(Class<?> clazz) {
        if (profiles == null) {
            profiles = new HashMap<>();
        }

        Profile profile = profiles.values().stream().filter(p -> p.getClass().equals(clazz)).findFirst().orElse(null);
        if (profile == null) {
            try {
                profile = (Profile) clazz.newInstance();
                profiles.put(profile.getProfileKey(), profile);
            } catch (InstantiationException | IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return (T) profile;
    }

    /**
     * Gets the guild as the JDA Guild
     *
     * @return the guild
     */
    public User getDiscordUser() {
        return DiscordService.JDA.getUserById(id);
    }
}
